@@ -4,7 +4,7 @@
  * Copyright (C) 2003, 2004 Red Hat, Inc.
  *
  * Licensed under the Academic Free License version 2.1
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -14,14 +14,18 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
+#include <config.h>
 #include "bus.h"
+
+#include <stdio.h>
+
 #include "activation.h"
 #include "connection.h"
 #include "services.h"
@@ -36,6 +40,10 @@
 #include <dbus/dbus-credentials.h>
 #include <dbus/dbus-internals.h>
 
+#ifdef DBUS_CYGWIN
+#include <signal.h>
+#endif
+
 struct BusContext
 {
   int refcount;
@@ -58,6 +66,8 @@ struct BusContext
   unsigned int fork : 1;
   unsigned int syslog : 1;
   unsigned int keep_umask : 1;
+  unsigned int allow_anonymous : 1;
+  unsigned int systemd_activation : 1;
 };
 
 static dbus_int32_t server_data_slot = -1;
@@ -74,7 +84,7 @@ server_get_context (DBusServer *server)
 {
   BusContext *context;
   BusServerData *bd;
-  
+
   if (!dbus_server_allocate_data_slot (&server_data_slot))
     return NULL;
 
@@ -101,7 +111,7 @@ server_watch_callback (DBusWatch     *watch,
    * if the code in activation.c for the babysitter
    * watch handler is fixed.
    */
-  
+
   return dbus_watch_handle (watch, condition);
 }
 
@@ -111,9 +121,9 @@ add_server_watch (DBusWatch  *watch,
 {
   DBusServer *server = data;
   BusContext *context;
-  
+
   context = server_get_context (server);
-  
+
   return _dbus_loop_add_watch (context->loop,
                                watch, server_watch_callback, server,
                                NULL);
@@ -125,9 +135,9 @@ remove_server_watch (DBusWatch  *watch,
 {
   DBusServer *server = data;
   BusContext *context;
-  
+
   context = server_get_context (server);
-  
+
   _dbus_loop_remove_watch (context->loop,
                            watch, server_watch_callback, server);
 }
@@ -147,7 +157,7 @@ add_server_timeout (DBusTimeout *timeout,
 {
   DBusServer *server = data;
   BusContext *context;
-  
+
   context = server_get_context (server);
 
   return _dbus_loop_add_timeout (context->loop,
@@ -160,9 +170,9 @@ remove_server_timeout (DBusTimeout *timeout,
 {
   DBusServer *server = data;
   BusContext *context;
-  
+
   context = server_get_context (server);
-  
+
   _dbus_loop_remove_timeout (context->loop,
                              timeout, server_timeout_callback, server);
 }
@@ -173,7 +183,7 @@ new_connection_callback (DBusServer     *server,
                          void           *data)
 {
   BusContext *context = data;
-  
+
   if (!bus_connections_setup_connection (context->connections, new_connection))
     {
       _dbus_verbose ("No memory to setup new connection\n");
@@ -191,15 +201,24 @@ new_connection_callback (DBusServer     *server,
 
   dbus_connection_set_max_message_size (new_connection,
                                         context->limits.max_message_size);
-  
+
+  dbus_connection_set_max_received_unix_fds (new_connection,
+                                         context->limits.max_incoming_unix_fds);
+
+  dbus_connection_set_max_message_unix_fds (new_connection,
+                                        context->limits.max_message_unix_fds);
+
+  dbus_connection_set_allow_anonymous (new_connection,
+                                       context->allow_anonymous);
+
   /* on OOM, we won't have ref'd the connection so it will die. */
 }
 
 static void
 free_server_data (void *data)
 {
-  BusServerData *bd = data;  
-  
+  BusServerData *bd = data;
+
   dbus_free (bd);
 }
 
@@ -222,17 +241,17 @@ setup_server (BusContext *context,
     }
 
   bd->context = context;
-  
+
   if (!dbus_server_set_auth_mechanisms (server, (const char**) auth_mechanisms))
     {
       BUS_SET_OOM (error);
       return FALSE;
     }
-  
+
   dbus_server_set_new_connection_function (server,
                                            new_connection_callback,
                                            context, NULL);
-  
+
   if (!dbus_server_set_watch_functions (server,
                                         add_server_watch,
                                         remove_server_watch,
@@ -253,7 +272,7 @@ setup_server (BusContext *context,
       BUS_SET_OOM (error);
       return FALSE;
     }
-  
+
   return TRUE;
 }
 
@@ -262,9 +281,11 @@ setup_server (BusContext *context,
  * when config files are reloaded.
  */
 static dbus_bool_t
-process_config_first_time_only (BusContext      *context,
-				BusConfigParser *parser,
-				DBusError       *error)
+process_config_first_time_only (BusContext       *context,
+				BusConfigParser  *parser,
+                                const DBusString *address,
+                                dbus_bool_t      systemd_activation,
+				DBusError        *error)
 {
   DBusString log_prefix;
   DBusList *link;
@@ -280,6 +301,10 @@ process_config_first_time_only (BusContext      *context,
   retval = FALSE;
   auth_mechanisms = NULL;
 
+  _dbus_init_system_log ();
+
+  context->systemd_activation = systemd_activation;
+
   /* Check for an existing pid file. Of course this is a race;
    * we'd have to use fcntl() locks on the pid file to
    * avoid that. But we want to check for the pid file
@@ -290,15 +315,34 @@ process_config_first_time_only (BusContext      *context,
     {
       DBusString u;
       DBusStat stbuf;
-      
+
       _dbus_string_init_const (&u, pidfile);
 
       if (_dbus_stat (&u, &stbuf, NULL))
         {
+#ifdef DBUS_CYGWIN
+          DBusString p;
+          long /* int */ pid;
+
+          _dbus_string_init (&p);
+          _dbus_file_get_contents(&p, &u, NULL);
+          _dbus_string_parse_int(&p, 0, &pid, NULL);
+          _dbus_string_free(&p);
+
+          if ((kill((int)pid, 0))) {
+            dbus_set_error(NULL, DBUS_ERROR_FILE_EXISTS,
+                           "pid %ld not running, removing stale pid file\n",
+                           pid);
+            _dbus_delete_file(&u, NULL);
+          } else {
+#endif
           dbus_set_error (error, DBUS_ERROR_FAILED,
 		                  "The pid file \"%s\" exists, if the message bus is not running, remove this file",
                           pidfile);
 	      goto failed;
+#ifdef DBUS_CYGWIN
+          }
+#endif
         }
     }
 
@@ -334,11 +378,20 @@ process_config_first_time_only (BusContext      *context,
       if (!credentials)
         goto oom;
       if (!_dbus_string_append (&log_prefix, "[session "))
-        goto oom;
+        {
+          _dbus_credentials_unref (credentials);
+          goto oom;
+        }
       if (!_dbus_credentials_to_string_append (credentials, &log_prefix))
-        goto oom;
+        {
+          _dbus_credentials_unref (credentials);
+          goto oom;
+        }
       if (!_dbus_string_append (&log_prefix, "] "))
-        goto oom;
+        {
+          _dbus_credentials_unref (credentials);
+          goto oom;
+        }
       _dbus_credentials_unref (credentials);
     }
   if (!_dbus_string_steal_data (&log_prefix, &context->log_prefix))
@@ -366,6 +419,7 @@ process_config_first_time_only (BusContext      *context,
           if (auth_mechanisms[i] == NULL)
             goto oom;
           link = _dbus_list_get_next_link (auth_mechanisms_list, link);
+          i += 1;
         }
     }
   else
@@ -374,35 +428,58 @@ process_config_first_time_only (BusContext      *context,
     }
 
   /* Listen on our addresses */
-  
-  addresses = bus_config_parser_get_addresses (parser);  
-  
-  link = _dbus_list_get_first_link (addresses);
-  while (link != NULL)
+
+  if (address)
     {
       DBusServer *server;
-      
-      server = dbus_server_listen (link->data, error);
+
+      server = dbus_server_listen (_dbus_string_get_const_data(address), error);
       if (server == NULL)
-	{
-	  _DBUS_ASSERT_ERROR_IS_SET (error);
-	  goto failed;
-	}
+        {
+          _DBUS_ASSERT_ERROR_IS_SET (error);
+          goto failed;
+        }
       else if (!setup_server (context, server, auth_mechanisms, error))
-	{
-	  _DBUS_ASSERT_ERROR_IS_SET (error);
-	  goto failed;
-	}
+        {
+          _DBUS_ASSERT_ERROR_IS_SET (error);
+          goto failed;
+        }
 
       if (!_dbus_list_append (&context->servers, server))
         goto oom;
+    }
+  else
+    {
+      addresses = bus_config_parser_get_addresses (parser);
 
-      link = _dbus_list_get_next_link (addresses, link);
+      link = _dbus_list_get_first_link (addresses);
+      while (link != NULL)
+        {
+          DBusServer *server;
+
+          server = dbus_server_listen (link->data, error);
+          if (server == NULL)
+            {
+              _DBUS_ASSERT_ERROR_IS_SET (error);
+              goto failed;
+            }
+          else if (!setup_server (context, server, auth_mechanisms, error))
+            {
+              _DBUS_ASSERT_ERROR_IS_SET (error);
+              goto failed;
+            }
+
+          if (!_dbus_list_append (&context->servers, server))
+            goto oom;
+
+          link = _dbus_list_get_next_link (addresses, link);
+        }
     }
 
   context->fork = bus_config_parser_get_fork (parser);
   context->syslog = bus_config_parser_get_syslog (parser);
   context->keep_umask = bus_config_parser_get_keep_umask (parser);
+  context->allow_anonymous = bus_config_parser_get_allow_anonymous (parser);
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
   retval = TRUE;
@@ -436,7 +513,7 @@ process_config_every_time (BusContext      *context,
   char *addr;
   const char *servicehelper;
   char *s;
-  
+
   dbus_bool_t retval;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
@@ -541,7 +618,7 @@ process_config_every_time (BusContext      *context,
 
  failed:
   _dbus_string_free (&full_address);
-  
+
   if (addr)
     dbus_free (addr);
 
@@ -557,7 +634,6 @@ list_concat_new (DBusList **a,
 
   *result = NULL;
 
-  link = _dbus_list_get_first_link (a);
   for (link = _dbus_list_get_first_link (a); link; link = _dbus_list_get_next_link (a, link))
     {
       if (!_dbus_list_append (result, link->data))
@@ -575,6 +651,24 @@ oom:
   return FALSE;
 }
 
+static void
+raise_file_descriptor_limit (BusContext      *context)
+{
+
+  /* I just picked this out of thin air; we need some extra
+   * descriptors for things like any internal pipes we create,
+   * inotify, connections to SELinux, etc.
+   */
+  unsigned int arbitrary_extra_fds = 32;
+  unsigned int limit;
+
+  limit = context->limits.max_completed_connections +
+    context->limits.max_incomplete_connections
+    + arbitrary_extra_fds;
+
+  _dbus_request_file_descriptor_limit (limit);
+}
+
 static dbus_bool_t
 process_config_postinit (BusContext      *context,
 			 BusConfigParser *parser,
@@ -583,6 +677,8 @@ process_config_postinit (BusContext      *context,
   DBusHashTable *service_context_table;
   DBusList *watched_dirs = NULL;
 
+  raise_file_descriptor_limit (context);
+
   service_context_table = bus_config_parser_steal_service_context_table (parser);
   if (!bus_registry_set_service_context_table (context->registry,
 					       service_context_table))
@@ -616,6 +712,8 @@ bus_context_new (const DBusString *config_file,
                  ForceForkSetting  force_fork,
                  DBusPipe         *print_addr_pipe,
                  DBusPipe         *print_pid_pipe,
+                 const DBusString *address,
+                 dbus_bool_t      systemd_activation,
                  DBusError        *error)
 {
   DBusString log_prefix;
@@ -669,8 +767,8 @@ bus_context_new (const DBusString *config_file,
       _DBUS_ASSERT_ERROR_IS_SET (error);
       goto failed;
     }
-  
-  if (!process_config_first_time_only (context, parser, error))
+
+  if (!process_config_first_time_only (context, parser, address, systemd_activation, error))
     {
       _DBUS_ASSERT_ERROR_IS_SET (error);
       goto failed;
@@ -680,7 +778,7 @@ bus_context_new (const DBusString *config_file,
       _DBUS_ASSERT_ERROR_IS_SET (error);
       goto failed;
     }
-  
+
   /* we need another ref of the server data slot for the context
    * to own
    */
@@ -697,14 +795,14 @@ bus_context_new (const DBusString *config_file,
       DBusString addr;
       const char *a = bus_context_get_address (context);
       int bytes;
-      
+
       _dbus_assert (a != NULL);
       if (!_dbus_string_init (&addr))
         {
           BUS_SET_OOM (error);
           goto failed;
         }
-      
+
       if (!_dbus_string_append (&addr, a) ||
           !_dbus_string_append (&addr, "\n"))
         {
@@ -728,10 +826,10 @@ bus_context_new (const DBusString *config_file,
 
       if (!_dbus_pipe_is_stdout_or_stderr (print_addr_pipe))
         _dbus_pipe_close (print_addr_pipe, NULL);
-      
+
       _dbus_string_free (&addr);
     }
-  
+
   context->connections = bus_connections_new (context);
   if (context->connections == NULL)
     {
@@ -768,8 +866,8 @@ bus_context_new (const DBusString *config_file,
     if ((force_fork != FORK_NEVER && context->fork) || force_fork == FORK_ALWAYS)
       {
         _dbus_verbose ("Forking and becoming daemon\n");
-        
-        if (!_dbus_become_daemon (context->pidfile ? &u : NULL, 
+
+        if (!_dbus_become_daemon (context->pidfile ? &u : NULL,
                                   print_pid_pipe,
                                   error,
                                   context->keep_umask))
@@ -781,7 +879,7 @@ bus_context_new (const DBusString *config_file,
     else
       {
         _dbus_verbose ("Fork not requested\n");
-        
+
         /* Need to write PID file and to PID pipe for ourselves,
          * not for the child process. This is a no-op if the pidfile
          * is NULL and print_pid_pipe is NULL.
@@ -817,7 +915,7 @@ bus_context_new (const DBusString *config_file,
       bus_config_parser_unref (parser);
       parser = NULL;
     }
-  
+
   /* Here we change our credentials if required,
    * as soon as we've set up our sockets and pidfile
    */
@@ -836,10 +934,10 @@ bus_context_new (const DBusString *config_file,
     }
 
   dbus_server_free_data_slot (&server_data_slot);
-  
+
   return context;
-  
- failed:  
+
+ failed:
   if (parser != NULL)
     bus_config_parser_unref (parser);
   if (context != NULL)
@@ -847,7 +945,7 @@ bus_context_new (const DBusString *config_file,
 
   if (server_data_slot >= 0)
     dbus_server_free_data_slot (&server_data_slot);
-  
+
   return NULL;
 }
 
@@ -877,7 +975,7 @@ bus_context_reload_config (BusContext *context,
       _DBUS_ASSERT_ERROR_IS_SET (error);
       goto failed;
     }
-  
+
   if (!process_config_every_time (context, parser, TRUE, error))
     {
       _DBUS_ASSERT_ERROR_IS_SET (error);
@@ -906,19 +1004,19 @@ shutdown_server (BusContext *context,
   if (server == NULL ||
       !dbus_server_get_is_connected (server))
     return;
-  
+
   if (!dbus_server_set_watch_functions (server,
                                         NULL, NULL, NULL,
                                         context,
                                         NULL))
     _dbus_assert_not_reached ("setting watch functions to NULL failed");
-  
+
   if (!dbus_server_set_timeout_functions (server,
                                           NULL, NULL, NULL,
                                           context,
                                           NULL))
     _dbus_assert_not_reached ("setting timeout functions to NULL failed");
-  
+
   dbus_server_disconnect (server);
 }
 
@@ -954,9 +1052,9 @@ bus_context_unref (BusContext *context)
   if (context->refcount == 0)
     {
       DBusList *link;
-      
+
       _dbus_verbose ("Finalizing bus context %p\n", context);
-      
+
       bus_context_shutdown (context);
 
       if (context->connections)
@@ -964,13 +1062,13 @@ bus_context_unref (BusContext *context)
           bus_connections_unref (context->connections);
           context->connections = NULL;
         }
-      
+
       if (context->registry)
         {
           bus_registry_unref (context->registry);
           context->registry = NULL;
         }
-      
+
       if (context->activation)
         {
           bus_activation_unref (context->activation);
@@ -981,7 +1079,7 @@ bus_context_unref (BusContext *context)
       while (link != NULL)
         {
           dbus_server_unref (link->data);
-          
+
           link = _dbus_list_get_next_link (&context->servers, link);
         }
       _dbus_list_clear (&context->servers);
@@ -991,7 +1089,7 @@ bus_context_unref (BusContext *context)
           bus_policy_unref (context->policy);
           context->policy = NULL;
         }
-      
+
       if (context->loop)
         {
           _dbus_loop_unref (context->loop);
@@ -1021,7 +1119,7 @@ bus_context_unref (BusContext *context)
 	   */
 	  _dbus_delete_file (&u, NULL);
 
-          dbus_free (context->pidfile); 
+          dbus_free (context->pidfile);
 	}
       dbus_free (context);
 
@@ -1048,6 +1146,12 @@ bus_context_get_servicehelper (BusContext *context)
   return context->servicehelper;
 }
 
+dbus_bool_t
+bus_context_get_systemd_activation (BusContext *context)
+{
+  return context->systemd_activation;
+}
+
 BusRegistry*
 bus_context_get_registry (BusContext  *context)
 {
@@ -1117,7 +1221,7 @@ bus_context_create_client_policy (BusContext      *context,
 int
 bus_context_get_activation_timeout (BusContext *context)
 {
-  
+
   return context->limits.activation_timeout;
 }
 
@@ -1184,7 +1288,14 @@ bus_context_log (BusContext *context, DBusSystemLogSeverity severity, const char
   va_list args;
 
   if (!context->syslog)
-    return;
+    {
+      /* we're not syslogging; just output to stderr */
+      va_start (args, msg);
+      vfprintf (stderr, msg, args);
+      fprintf (stderr, "\n");
+      va_end (args);
+      return;
+    }
 
   va_start (args, msg);
 
@@ -1210,6 +1321,77 @@ out:
   va_end (args);
 }
 
+static inline const char *
+nonnull (const char *maybe_null,
+         const char *if_null)
+{
+  return (maybe_null ? maybe_null : if_null);
+}
+
+/*
+ * Log something about a message, usually that it was rejected.
+ */
+static void
+complain_about_message (BusContext     *context,
+                        const char     *error_name,
+                        const char     *complaint,
+                        int             matched_rules,
+                        DBusMessage    *message,
+                        DBusConnection *sender,
+                        DBusConnection *proposed_recipient,
+                        dbus_bool_t     requested_reply,
+                        dbus_bool_t     log,
+                        DBusError      *error)
+{
+  DBusError stack_error = DBUS_ERROR_INIT;
+  const char *sender_name;
+  const char *sender_loginfo;
+  const char *proposed_recipient_loginfo;
+
+  if (error == NULL && !log)
+    return;
+
+  if (sender != NULL)
+    {
+      sender_name = bus_connection_get_name (sender);
+      sender_loginfo = bus_connection_get_loginfo (sender);
+    }
+  else
+    {
+      sender_name = "(unset)";
+      sender_loginfo = "(bus)";
+    }
+
+  if (proposed_recipient != NULL)
+    proposed_recipient_loginfo = bus_connection_get_loginfo (proposed_recipient);
+  else
+    proposed_recipient_loginfo = "bus";
+
+  dbus_set_error (&stack_error, error_name,
+      "%s, %d matched rules; type=\"%s\", sender=\"%s\" (%s) "
+      "interface=\"%s\" member=\"%s\" error name=\"%s\" "
+      "requested_reply=\"%d\" destination=\"%s\" (%s)",
+      complaint,
+      matched_rules,
+      dbus_message_type_to_string (dbus_message_get_type (message)),
+      sender_name,
+      sender_loginfo,
+      nonnull (dbus_message_get_interface (message), "(unset)"),
+      nonnull (dbus_message_get_member (message), "(unset)"),
+      nonnull (dbus_message_get_error_name (message), "(unset)"),
+      requested_reply,
+      nonnull (dbus_message_get_destination (message), DBUS_SERVICE_DBUS),
+      proposed_recipient_loginfo);
+
+  /* If we hit OOM while setting the error, this will syslog "out of memory"
+   * which is itself an indication that something is seriously wrong */
+  if (log)
+    bus_context_log (context, DBUS_SYSTEM_LOG_SECURITY, "%s",
+        stack_error.message);
+
+  dbus_move_error (&stack_error, error);
+}
+
 /*
  * addressed_recipient is the recipient specified in the message.
  *
@@ -1242,10 +1424,10 @@ bus_context_check_security_policy (BusContext     *context,
   const char *sender_name;
   const char *sender_loginfo;
   const char *proposed_recipient_loginfo;
-  
+
   type = dbus_message_get_type (message);
   dest = dbus_message_get_destination (message);
-  
+
   /* dispatch.c was supposed to ensure these invariants */
   _dbus_assert (dest != NULL ||
                 type == DBUS_MESSAGE_TYPE_SIGNAL ||
@@ -1254,23 +1436,6 @@ bus_context_check_security_policy (BusContext     *context,
                 addressed_recipient != NULL ||
                 strcmp (dest, DBUS_SERVICE_DBUS) == 0);
 
-  /* Used in logging below */
-  if (sender != NULL)
-    {
-      sender_name = bus_connection_get_name (sender);
-      sender_loginfo = bus_connection_get_loginfo (sender);
-    }
-  else
-    {
-      sender_name = NULL;
-      sender_loginfo = "(bus)";
-    }
-  
-  if (proposed_recipient != NULL)
-    proposed_recipient_loginfo = bus_connection_get_loginfo (proposed_recipient);
-  else
-    proposed_recipient_loginfo = "bus";
-  
   switch (type)
     {
     case DBUS_MESSAGE_TYPE_METHOD_CALL:
@@ -1278,19 +1443,19 @@ bus_context_check_security_policy (BusContext     *context,
     case DBUS_MESSAGE_TYPE_METHOD_RETURN:
     case DBUS_MESSAGE_TYPE_ERROR:
       break;
-      
+
     default:
       _dbus_verbose ("security check disallowing message of unknown type %d\n",
                      type);
 
       dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED,
                       "Message bus will not accept messages of unknown type\n");
-              
+
       return FALSE;
     }
 
   requested_reply = FALSE;
-  
+
   if (sender != NULL)
     {
       /* First verify the SELinux access controls.  If allowed then
@@ -1305,30 +1470,23 @@ bus_context_check_security_policy (BusContext     *context,
         {
           if (error != NULL && !dbus_error_is_set (error))
             {
-              dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED,
-                              "An SELinux policy prevents this sender "
-                              "from sending this message to this recipient "
-                              "(rejected message had sender \"%s\" interface \"%s\" "
-                              "member \"%s\" error name \"%s\" destination \"%s\")",
-                              sender_name ? sender_name : "(unset)",
-                              dbus_message_get_interface (message) ?
-                              dbus_message_get_interface (message) : "(unset)",
-                              dbus_message_get_member (message) ?
-                              dbus_message_get_member (message) : "(unset)",
-                              dbus_message_get_error_name (message) ?
-                              dbus_message_get_error_name (message) : "(unset)",
-                              dest ? dest : DBUS_SERVICE_DBUS);
+              /* don't syslog this, just set the error: avc_has_perm should
+               * have already written to either the audit log or syslog */
+              complain_about_message (context, DBUS_ERROR_ACCESS_DENIED,
+                  "An SELinux policy prevents this sender from sending this "
+                  "message to this recipient",
+                  0, message, sender, proposed_recipient, FALSE, FALSE, error);
               _dbus_verbose ("SELinux security check denying send to service\n");
             }
 
           return FALSE;
         }
-       
+
       if (bus_connection_is_active (sender))
         {
           sender_policy = bus_connection_get_policy (sender);
           _dbus_assert (sender_policy != NULL);
-          
+
           /* Fill in requested_reply variable with TRUE if this is a
            * reply and the reply was pending.
            */
@@ -1337,8 +1495,8 @@ bus_context_check_security_policy (BusContext     *context,
               if (proposed_recipient != NULL /* not to the bus driver */ &&
                   addressed_recipient == proposed_recipient /* not eavesdropping */)
                 {
-                  DBusError error2;                  
-                  
+                  DBusError error2;
+
                   dbus_error_init (&error2);
                   requested_reply = bus_connections_check_reply (bus_connection_get_connections (sender),
                                                                  transaction,
@@ -1374,7 +1532,7 @@ bus_context_check_security_policy (BusContext     *context,
               dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED,
                               "Client tried to send a message other than %s without being registered",
                               "Hello");
-              
+
               return FALSE;
             }
         }
@@ -1393,7 +1551,7 @@ bus_context_check_security_policy (BusContext     *context,
 
   _dbus_assert ((sender != NULL && sender_policy != NULL) ||
                 (sender == NULL && sender_policy == NULL));
-  
+
   if (proposed_recipient != NULL)
     {
       /* only the bus driver can send to an inactive recipient (as it
@@ -1418,11 +1576,11 @@ bus_context_check_security_policy (BusContext     *context,
     }
   else
     recipient_policy = NULL;
-  
+
   _dbus_assert ((proposed_recipient != NULL && recipient_policy != NULL) ||
                 (proposed_recipient != NULL && sender == NULL && recipient_policy == NULL) ||
                 (proposed_recipient == NULL && recipient_policy == NULL));
-  
+
   log = FALSE;
   if (sender_policy &&
       !bus_client_policy_check_can_send (sender_policy,
@@ -1434,57 +1592,23 @@ bus_context_check_security_policy (BusContext     *context,
       const char *msg = "Rejected send message, %d matched rules; "
                         "type=\"%s\", sender=\"%s\" (%s) interface=\"%s\" member=\"%s\" error name=\"%s\" requested_reply=%d destination=\"%s\" (%s))";
 
-      dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED, msg,
-                      toggles,
-                      dbus_message_type_to_string (dbus_message_get_type (message)),
-                      sender_name ? sender_name : "(unset)",
-                      sender_loginfo,
-                      dbus_message_get_interface (message) ?
-                      dbus_message_get_interface (message) : "(unset)",
-                      dbus_message_get_member (message) ?
-                      dbus_message_get_member (message) : "(unset)",
-                      dbus_message_get_error_name (message) ?
-                      dbus_message_get_error_name (message) : "(unset)",
-                      requested_reply,
-                      dest ? dest : DBUS_SERVICE_DBUS,
-                      proposed_recipient_loginfo);
-      /* Needs to be duplicated to avoid calling malloc and having to handle OOM */
-      if (addressed_recipient == proposed_recipient)
-        bus_context_log (context, DBUS_SYSTEM_LOG_SECURITY, msg,
-                                  toggles,
-                                  dbus_message_type_to_string (dbus_message_get_type (message)),
-                                  sender_name ? sender_name : "(unset)",
-                                  sender_loginfo,
-                                  dbus_message_get_interface (message) ?
-                                  dbus_message_get_interface (message) : "(unset)",
-                                  dbus_message_get_member (message) ?
-                                  dbus_message_get_member (message) : "(unset)",
-                                  dbus_message_get_error_name (message) ?
-                                  dbus_message_get_error_name (message) : "(unset)",
-                                  requested_reply,
-                                  dest ? dest : DBUS_SERVICE_DBUS,
-                                  proposed_recipient_loginfo);
+      complain_about_message (context, DBUS_ERROR_ACCESS_DENIED,
+          "Rejected send message", toggles,
+          message, sender, proposed_recipient, requested_reply,
+          (addressed_recipient == proposed_recipient), error);
       _dbus_verbose ("security policy disallowing message due to sender policy\n");
       return FALSE;
     }
 
   if (log)
-    bus_context_log (context, DBUS_SYSTEM_LOG_SECURITY,
-                              "Would reject message, %d matched rules; "
-                              "type=\"%s\", sender=\"%s\" (%s) interface=\"%s\" member=\"%s\" error name=\"%s\" requested_reply=%d destination=\"%s\" (%s))",
-                              toggles,
-                              dbus_message_type_to_string (dbus_message_get_type (message)),
-                              sender_name ? sender_name : "(unset)",
-                              sender_loginfo,
-                              dbus_message_get_interface (message) ?
-                              dbus_message_get_interface (message) : "(unset)",
-                              dbus_message_get_member (message) ?
-                              dbus_message_get_member (message) : "(unset)",
-                              dbus_message_get_error_name (message) ?
-                              dbus_message_get_error_name (message) : "(unset)",
-                              requested_reply,                               
-                              dest ? dest : DBUS_SERVICE_DBUS,
-                              proposed_recipient_loginfo);
+    {
+      /* We want to drop this message, and are only not doing so for backwards
+       * compatibility. */
+      complain_about_message (context, DBUS_ERROR_ACCESS_DENIED,
+          "Would reject message", toggles,
+          message, sender, proposed_recipient, requested_reply,
+          TRUE, NULL);
+    }
 
   if (recipient_policy &&
       !bus_client_policy_check_can_receive (recipient_policy,
@@ -1494,55 +1618,23 @@ bus_context_check_security_policy (BusContext     *context,
                                             addressed_recipient, proposed_recipient,
                                             message, &toggles))
     {
-      const char *msg = "Rejected receive message, %d matched rules; "
-                        "type=\"%s\" sender=\"%s\" (%s) interface=\"%s\" member=\"%s\" error name=\"%s\" reply serial=%u requested_reply=%d destination=\"%s\" (%s))";
-
-      dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED, msg,
-                      toggles,
-                      dbus_message_type_to_string (dbus_message_get_type (message)),
-                      sender_name ? sender_name : "(unset)",
-                      sender_loginfo,
-                      dbus_message_get_interface (message) ?
-                      dbus_message_get_interface (message) : "(unset)",
-                      dbus_message_get_member (message) ?
-                      dbus_message_get_member (message) : "(unset)",
-                      dbus_message_get_error_name (message) ?
-                      dbus_message_get_error_name (message) : "(unset)",
-                      dbus_message_get_reply_serial (message),
-                      requested_reply,
-                      dest ? dest : DBUS_SERVICE_DBUS,
-                      proposed_recipient_loginfo);
-      /* Needs to be duplicated to avoid calling malloc and having to handle OOM */
-      if (addressed_recipient == proposed_recipient)
-        bus_context_log (context, DBUS_SYSTEM_LOG_SECURITY, msg,
-                                  toggles,
-                                  dbus_message_type_to_string (dbus_message_get_type (message)),
-                                  sender_name ? sender_name : "(unset)",
-                                  sender_loginfo,
-                                  dbus_message_get_interface (message) ?
-                                  dbus_message_get_interface (message) : "(unset)",
-                                  dbus_message_get_member (message) ?
-                                  dbus_message_get_member (message) : "(unset)",
-                                  dbus_message_get_error_name (message) ?
-                                  dbus_message_get_error_name (message) : "(unset)",
-                                  dbus_message_get_reply_serial (message),
-                                  requested_reply,
-                                  dest ? dest : DBUS_SERVICE_DBUS,
-                                  proposed_recipient_loginfo);
+      complain_about_message (context, DBUS_ERROR_ACCESS_DENIED,
+          "Rejected receive message", toggles,
+          message, sender, proposed_recipient, requested_reply,
+          (addressed_recipient == proposed_recipient), NULL);
       _dbus_verbose ("security policy disallowing message due to recipient policy\n");
       return FALSE;
     }
 
   /* See if limits on size have been exceeded */
   if (proposed_recipient &&
-      dbus_connection_get_outgoing_size (proposed_recipient) >
-      context->limits.max_outgoing_bytes)
+      ((dbus_connection_get_outgoing_size (proposed_recipient) > context->limits.max_outgoing_bytes) ||
+       (dbus_connection_get_outgoing_unix_fds (proposed_recipient) > context->limits.max_outgoing_unix_fds)))
     {
-      dbus_set_error (error, DBUS_ERROR_LIMITS_EXCEEDED,
-                      "The destination service \"%s\" has a full message queue",
-                      dest ? dest : (proposed_recipient ?
-                                     bus_connection_get_name (proposed_recipient) : 
-                                     DBUS_SERVICE_DBUS));
+      complain_about_message (context, DBUS_ERROR_LIMITS_EXCEEDED,
+          "Rejected: destination has a full message queue",
+          0, message, sender, proposed_recipient, requested_reply, TRUE,
+          error);
       _dbus_verbose ("security policy disallowing message due to full message queue\n");
       return FALSE;
     }
@@ -1552,7 +1644,7 @@ bus_context_check_security_policy (BusContext     *context,
    * connection). Only the addressed recipient may reply.
    */
   if (type == DBUS_MESSAGE_TYPE_METHOD_CALL &&
-      sender && 
+      sender &&
       addressed_recipient &&
       addressed_recipient == proposed_recipient && /* not eavesdropping */
       !bus_connections_expect_reply (bus_connection_get_connections (sender),
@@ -1563,7 +1655,7 @@ bus_context_check_security_policy (BusContext     *context,
       _dbus_verbose ("Failed to record reply expectation or problem with the message expecting a reply\n");
       return FALSE;
     }
-  
+
   _dbus_verbose ("security policy allowing message\n");
   return TRUE;
 }
