@@ -164,7 +164,7 @@ bus_config_parser_start_element (BusConfigParser   *parser,
     {
     case ELEMENT_SERVICEHELPER:
     case ELEMENT_USER:
-    case ELEMENT_TYPE:
+    case ELEMENT_CONFIGTYPE:
       /* content about to be handled */
       break;
 
@@ -188,7 +188,7 @@ bus_config_parser_start_element (BusConfigParser   *parser,
     default:
       {
         /* we really don't care about the others... */
-        _dbus_verbose (" START We dont care about '%s' type '%i'\n", element_name, parser->type);
+        _dbus_verbose (" START We don't care about '%s' type '%i'\n", element_name, parser->type);
         break;
       }
     }
@@ -277,7 +277,7 @@ bus_config_parser_content (BusConfigParser   *parser,
       }
       break;
 
-    case ELEMENT_TYPE:
+    case ELEMENT_CONFIGTYPE:
       {
         if (!_dbus_string_copy (&content_sane, 0, &parser->bus_type, 0))
           {
@@ -289,7 +289,7 @@ bus_config_parser_content (BusConfigParser   *parser,
     default:
       {
         /* we don't care about the others... really */
-        _dbus_verbose (" CONTENTS We dont care '%s' type '%i'\n", _dbus_string_get_const_data (&content_sane), parser->type);
+        _dbus_verbose (" CONTENTS We don't care about '%s' type '%i'\n", _dbus_string_get_const_data (&content_sane), parser->type);
         break;
       }
     }
